@@ -1,44 +1,11 @@
-/*@HEADER
-// ***********************************************************************
-//
+// @HEADER
+// *****************************************************************************
 //       Ifpack2: Templated Object-Oriented Algebraic Preconditioner Package
-//                 Copyright (2009) Sandia Corporation
 //
-// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
-// license for use of this work by or on behalf of the U.S. Government.
-//
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-// 1. Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//
-// 2. Redistributions in binary form must reproduce the above copyright
-// notice, this list of conditions and the following disclaimer in the
-// documentation and/or other materials provided with the distribution.
-//
-// 3. Neither the name of the Corporation nor the names of the
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
-// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
-// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-//
-// Questions? Contact Michael A. Heroux (maherou@sandia.gov)
-//
-// ***********************************************************************
-//@HEADER
-*/
+// Copyright 2009 NTESS and the Ifpack2 contributors.
+// SPDX-License-Identifier: BSD-3-Clause
+// *****************************************************************************
+// @HEADER
 
 #ifndef IFPACK2_BLOCKRELAXATION_DEF_HPP
 #define IFPACK2_BLOCKRELAXATION_DEF_HPP
@@ -53,6 +20,8 @@
 #include "Ifpack2_LocalFilter.hpp"
 #include "Ifpack2_Parameters.hpp"
 #include "Teuchos_TimeMonitor.hpp"
+#include "Tpetra_BlockCrsMatrix_Helpers_decl.hpp"
+#include "Tpetra_Import_Util.hpp"
 
 namespace Ifpack2 {
 
@@ -176,6 +145,10 @@ getValidParameters () const
   validParams->set("partitioner: coordinates",dummy);
   validParams->set("timer for apply", true);
   validParams->set("partitioner: subparts per part", 1);
+  validParams->set("partitioner: block size", -1);
+  validParams->set("partitioner: print level", false);
+  validParams->set("partitioner: explicit convert to BlockCrs", false);
+  validParams->set("partitioner: checkBlockConsistency", true);
 
   return validParams;
 }
@@ -629,11 +602,32 @@ initialize ()
     Teuchos::RCP<const block_crs_matrix_type> A_bcrs =
       Teuchos::rcp_dynamic_cast<const block_crs_matrix_type> (A_);
     hasBlockCrsMatrix_ = !A_bcrs.is_null();
-    if (A_bcrs.is_null ()) {
-      hasBlockCrsMatrix_ = false;
-    }
-    else {
-      hasBlockCrsMatrix_ = true;
+
+    Teuchos::RCP<const row_graph_type> graph = A_->getGraph ();
+
+    if(!hasBlockCrsMatrix_ && List_.isParameter("relaxation: container") && List_.get<std::string>("relaxation: container") == "BlockTriDi" ) {
+      TEUCHOS_FUNC_TIME_MONITOR("Ifpack2::BlockRelaxation::initialize::convertToBlockCrsMatrix");
+      int block_size = List_.get<int>("partitioner: block size");
+      bool use_explicit_conversion = List_.isParameter("partitioner: explicit convert to BlockCrs") && List_.get<bool>("partitioner: explicit convert to BlockCrs");
+      TEUCHOS_TEST_FOR_EXCEPT_MSG
+        (use_explicit_conversion && block_size == -1, "A pointwise matrix and block_size = -1 were given as inputs.");
+      if(use_explicit_conversion) {
+        A_bcrs = Tpetra::convertToBlockCrsMatrix(*Teuchos::rcp_dynamic_cast<const crs_matrix_type>(A_), block_size, false);
+        A_ = A_bcrs;
+        hasBlockCrsMatrix_ = true;
+        graph = A_->getGraph ();
+      }
+      else {
+        if ( !List_.isParameter("partitioner: checkBlockConsistency") || List_.get<bool>("partitioner: checkBlockConsistency")) {
+          if ( !A_->getGraph ()->getImporter().is_null()) {
+            TEUCHOS_TEST_FOR_EXCEPT_MSG
+              (!Tpetra::Import_Util::checkBlockConsistency(*(A_->getGraph ()->getColMap()), block_size), 
+              "The pointwise graph of the input matrix A pointwise is not consistent with block_size.");
+          }
+        }
+        graph = Tpetra::getBlockCrsGraph(*Teuchos::rcp_dynamic_cast<const crs_matrix_type>(A_), block_size, true);
+      }
+      Kokkos::DefaultExecutionSpace().fence();
     }
 
     NumLocalRows_      = A_->getLocalNumRows ();
@@ -647,19 +641,19 @@ initialize ()
 
     if (PartitionerType_ == "linear") {
       Partitioner_ =
-        rcp (new Ifpack2::LinearPartitioner<row_graph_type> (A_->getGraph ()));
+        rcp (new Ifpack2::LinearPartitioner<row_graph_type> (graph));
     } else if (PartitionerType_ == "line") {
       Partitioner_ =
-        rcp (new Ifpack2::LinePartitioner<row_graph_type,typename MatrixType::scalar_type> (A_->getGraph ()));
+        rcp (new Ifpack2::LinePartitioner<row_graph_type,typename MatrixType::scalar_type> (graph));
     } else if (PartitionerType_ == "user") {
       Partitioner_ =
-        rcp (new Ifpack2::Details::UserPartitioner<row_graph_type> (A_->getGraph () ) );
+        rcp (new Ifpack2::Details::UserPartitioner<row_graph_type> (graph ) );
     } else if (PartitionerType_ == "zoltan2") {
       #if defined(HAVE_IFPACK2_ZOLTAN2)
-      if (A_->getGraph ()->getComm ()->getSize () == 1) {
+      if (graph->getComm ()->getSize () == 1) {
         // Only one MPI, so call zoltan2 with global graph
         Partitioner_ =
-          rcp (new Ifpack2::Zoltan2Partitioner<row_graph_type> (A_->getGraph ()) );
+          rcp (new Ifpack2::Zoltan2Partitioner<row_graph_type> (graph) );
       } else {
         // Form local matrix to get local graph for calling zoltan2
         Teuchos::RCP<const row_matrix_type> A_local = rcp (new LocalFilter<row_matrix_type> (A_));
