@@ -75,7 +75,11 @@ public:
   void operator()(const int elem) const
   {
     for(int i=0;i<static_cast<int>(points.extent(1));i++) {
-      auto x = points(elem,i,0), y = points(elem,i,1), z = points(elem,i,2);
+      auto x = points(elem,i,0);
+      auto y = points(elem,i,1);
+      double z = 0.;
+      if (points.extent(2) == 3)
+        z = points(elem,i,2);
       if (funAtPoints.rank() == 3) {
         for(int d=0;d<static_cast<int>(funAtPoints.extent(2));d++)
           funAtPoints(elem,i,d) = fun(x,y,z,d); // vector basis
@@ -121,11 +125,7 @@ class GetCoeffsEvaluator
     DynRankView local_physEvalPoints;
     DynRankView local_refTargetAtEvalPoints, local_physTargetAtEvalPoints;
     ElemShape elemShape;
-  #ifdef HAVE_INTREPID2_EXPERIMENTAL_NAMESPACE
-    Intrepid2::Experimental::ProjectionStruct<PHX::Device,double> projStruct;
-  #else
     Intrepid2::ProjectionStruct<PHX::Device,double> projStruct;
-  #endif
     Teuchos::RCP<Intrepid2::Basis<PHX::Device::execution_space,double,double> > it2basis;
   
 }; // end of class
@@ -202,11 +202,7 @@ evaluateFields(
 {
   using ct = Intrepid2::CellTools<PHX::Device>;
   using fst = Intrepid2::FunctionSpaceTools<PHX::Device>;
-  #ifdef HAVE_INTREPID2_EXPERIMENTAL_NAMESPACE
-  using pts = Intrepid2::Experimental::ProjectionTools<PHX::Device>;
-  #else
   using pts = Intrepid2::ProjectionTools<PHX::Device>;
-  #endif 
 
   // FYI, this all relies on a first-order mesh
   auto cellNodesAll = workset.getCellNodes();
