@@ -1,4 +1,4 @@
-// Copyright(C) 1999-2022 National Technology & Engineering Solutions
+// Copyright(C) 1999-2023 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 //
@@ -17,15 +17,13 @@ public:
   Face_Block(int file_id, size_t id);
   Face_Block(int file_id, size_t id, size_t ne);
   ~Face_Block() override;
-
-  size_t Face_Index(size_t position) const;
-
-  int Check_State() const;
-
-private:
   Face_Block(const Face_Block &)                  = delete; // Not written.
   const Face_Block &operator=(const Face_Block &) = delete; // Not written.
 
+  size_t Face_Index(size_t position) const;
+
+private:
+  int  Check_State() const override;
   void entity_load_params() override;
 
   EXOTYPE     exodus_type() const override;
