@@ -102,10 +102,6 @@ public:
   KOKKOS_INLINE_FUNCTION
   void device_stale_access_check(NgpField *, const stk::mesh::FastMeshIndex &, int, const char *, int) const {}
 
-  template <typename NgpField>
-  KOKKOS_INLINE_FUNCTION
-  void device_stale_access_check(NgpField *, const stk::mesh::DeviceMesh::MeshIndex &, int, const char *, int) const {}
-
   template <typename NgpField>
   KOKKOS_INLINE_FUNCTION
   void device_stale_access_check(NgpField *, const stk::mesh::FastMeshIndex &, const char *, int) const {}
@@ -230,23 +226,6 @@ public:
     }
   }
 
-  template <typename NgpField>
-  KOKKOS_FUNCTION
-  void device_stale_access_check(NgpField* ngpField, const stk::mesh::DeviceMesh::MeshIndex& index, int component,
-                                 const char* fileName, int lineNumber) const
-  {
-    anyPotentialDeviceFieldModification() = true;
-
-    if (field_not_updated_after_mesh_mod(ngpField->synchronizedCount)) {
-      print_unupdated_field_warning(fileName, lineNumber);
-      return;
-    }
-
-    if (data_is_stale_on_device(index, component)) {
-      print_stale_data_warning(ngpField, index.bucket->bucket_id(), index.bucketOrd, component, fileName, lineNumber);
-    }
-  }
-
   template <typename NgpField>
   KOKKOS_FUNCTION
   void device_stale_access_check(NgpField* ngpField, const stk::mesh::FastMeshIndex& index,
@@ -439,11 +418,6 @@ private:
     return data_is_stale_on_device(index.bucket_id, index.bucket_ord, component);
   }
 
-  KOKKOS_INLINE_FUNCTION
-  bool data_is_stale_on_device(const stk::mesh::DeviceMesh::MeshIndex & index, int component) const {
-    return data_is_stale_on_device(index.bucket->bucket_id(), index.bucketOrd, component);
-  }
-
   KOKKOS_INLINE_FUNCTION
   bool data_is_stale_on_device(int bucketId, int bucketOrdinal, int component) const {
     return !(lastFieldModLocation(debugDeviceSelectedBucketOffset(bucketId),
