@@ -50,6 +50,7 @@ bool MeshModification::modification_begin(const std::string description)
     const stk::mesh::FieldVector allFields = m_bulkData.mesh_meta_data().get_fields();
     for (FieldBase * stkField : allFields) {
       stkField->sync_to_host();
+      stkField->modify_on_host();
       if (stkField->has_ngp_field()) {
         impl::get_ngp_field(*stkField)->debug_modification_begin();
       }
@@ -475,7 +476,7 @@ void MeshModification::delete_shared_entities_which_are_no_longer_in_owned_closu
   {
     Entity entity = i->entity;
     if (m_bulkData.is_valid(entity) && !m_bulkData.owned_closure(entity)) {
-      if (m_bulkData.in_shared(entity)) {
+      if (i->entity_comm != -1 && m_bulkData.in_shared(entity)) {
         destroy_dependent_ghosts(entity, entitiesToRemoveFromSharing, auraEntitiesToDestroy);
       }
     }
