@@ -52,7 +52,7 @@ using stk::unit_test_util::build_mesh;
 inline void findBoundingBoxCoordinates(const std::vector<double> &coordinates, std::vector<double>& boxCoordinates)
 {
     int spatialDim = 3;
-    double *minCoordinates = &boxCoordinates[0];
+    double *minCoordinates = boxCoordinates.data();
     double *maxCoordinates = &boxCoordinates[spatialDim];
 
     for (int j=0;j<spatialDim;j++)
@@ -219,7 +219,7 @@ inline void putCoordinatesInFile(const int exoid, const std::vector<FloatBox>& b
         double zmax = boxes[i].get_z_max();
 
         double hexCoordinates[24];
-        setHexCoordinates(xmin, ymin, zmin, xmax, ymax, zmax, &hexCoordinates[0]);
+        setHexCoordinates(xmin, ymin, zmin, xmax, ymax, zmax, hexCoordinates);
 
         unsigned offset = i*num_nodes_per_element;
         for (int j=0; j<num_nodes_per_element; j++)
@@ -281,7 +281,7 @@ inline void writeExodusFileUsingBoxes(const std::vector<FloatBox>& boxes, const
         }
         offset += num_elements_this_block*num_nodes_per_elem;
 
-        ex_put_elem_conn(exoid, blockId, &connect[0]);
+        ex_put_elem_conn(exoid, blockId, connect.data());
     }
 
     ex_close(exoid);
@@ -303,35 +303,48 @@ fillDomainBoxes(MPI_Comm comm)
     return domainBoxes;
 }
 
-inline void fillStkBoxesUsingFloatBoxes(const std::vector<FloatBox> &domainBoxes, const int procId, StkBoxVector& stkBoxes)
+inline void convertFloatBoxesToDoubleBoxes(const FloatBoxIdentProcVector &floatBoxes, StkBoxIdentProcVector& doubleBoxes)
+{
+  doubleBoxes.resize(floatBoxes.size());
+  for (size_t i=0;i<floatBoxes.size();i++) {
+    Point min(floatBoxes[i].first.get_x_min(), floatBoxes[i].first.get_y_min(), floatBoxes[i].first.get_z_min());
+    Point max(floatBoxes[i].first.get_x_max(), floatBoxes[i].first.get_y_max(), floatBoxes[i].first.get_z_max());
+    doubleBoxes[i] = std::make_pair(StkBox(min,max), floatBoxes[i].second);
+  }
+}
+
+inline void fillStkBoxesUsingFloatBoxes(const std::vector<FloatBox> &domainBoxes, const int procId, StkBoxIdentProcVector& stkBoxes)
 {
     for (size_t i=0;i<domainBoxes.size();i++)
     {
         Point min(domainBoxes[i].get_x_min(), domainBoxes[i].get_y_min(), domainBoxes[i].get_z_min());
         Point max(domainBoxes[i].get_x_max(), domainBoxes[i].get_y_max(), domainBoxes[i].get_z_max());
-        Ident domainBoxId(i, procId);
+        IdentProc domainBoxId(i, procId);
         stkBoxes[i] = std::make_pair(StkBox(min,max), domainBoxId);
     }
 }
 
-inline void createBoundingBoxesForElementsInElementBlocks(const stk::mesh::BulkData &bulk, FloatBoxVector& domainBoxes)
+template<typename BoxType, typename IdentProcType>
+inline void createBoundingBoxesForEntities(const stk::mesh::BulkData &bulk,
+                                           stk::mesh::EntityRank rank,
+                                           std::vector<std::pair<BoxType,IdentProcType>>& boundingBoxes)
 {
-    stk::mesh::EntityVector elements;
+    stk::mesh::EntityVector entities;
     const bool sortById = true;
-    stk::mesh::get_entities(bulk, stk::topology::ELEM_RANK, bulk.mesh_meta_data().locally_owned_part(), elements, sortById);
+    stk::mesh::get_entities(bulk, rank, bulk.mesh_meta_data().locally_owned_part(), entities, sortById);
 
-    size_t numberBoundingBoxes = elements.size();
-    domainBoxes.resize(numberBoundingBoxes);
+    size_t numberBoundingBoxes = entities.size();
+    boundingBoxes.resize(numberBoundingBoxes);
 
     stk::mesh::FieldBase const * coords = bulk.mesh_meta_data().coordinate_field();
 
     std::vector<double> boxCoordinates(6);
 
-    for(size_t i=0;i<elements.size();++i)
+    for(size_t i=0;i<entities.size();++i)
     {
-        unsigned num_nodes = bulk.num_nodes(elements[i]);
+        unsigned num_nodes = bulk.num_nodes(entities[i]);
         std::vector<double> coordinates(3*num_nodes,0);
-        const stk::mesh::Entity* nodes = bulk.begin_nodes(elements[i]);
+        const stk::mesh::Entity* nodes = bulk.begin_nodes(entities[i]);
         for(unsigned j=0;j<num_nodes;++j)
         {
             double* data = static_cast<double*>(stk::mesh::field_data(*coords, nodes[j]));
@@ -340,16 +353,77 @@ inline void createBoundingBoxesForElementsInElementBlocks(const stk::mesh::BulkD
             coordinates[3*j+2] = data[2];
         }
         findBoundingBoxCoordinates(coordinates, boxCoordinates);
-        unsigned id = bulk.identifier(elements[i]);
-        Ident domainBoxId(id, bulk.parallel_rank());
-        domainBoxes[i] = std::make_pair(FloatBox(boxCoordinates[0], boxCoordinates[1], boxCoordinates[2],
-                                                boxCoordinates[3], boxCoordinates[4], boxCoordinates[5]),
-                                                domainBoxId);
+        unsigned id = bulk.identifier(entities[i]);
+        IdentProcType domainBoxId;
+        if constexpr (std::is_same_v<IdentProcType, IdentProc>) {
+          domainBoxId = IdentProc(id, bulk.parallel_rank());
+        }
+        else {
+            domainBoxId = id;
+        }
+        boundingBoxes[i] = std::make_pair(BoxType(boxCoordinates[0], boxCoordinates[1], boxCoordinates[2],
+                                                  boxCoordinates[3], boxCoordinates[4], boxCoordinates[5]),
+                                                  domainBoxId);
 
     }
 }
 
-inline void fillBoxesUsingElementBlocksFromFile(MPI_Comm comm, const std::string& volumeFilename, FloatBoxVector &domainBoxes)
+template<typename BoxIdentProcType>
+inline Kokkos::View<BoxIdentProcType *>
+createBoundingBoxesForEntities(const stk::mesh::BulkData &bulk,
+                                     stk::mesh::EntityRank rank)
+{
+
+  using BoxType = typename BoxIdentProcType::box_type;  
+  using IdentProcType = typename BoxIdentProcType::second_type;  
+  stk::mesh::EntityVector entities;
+  const bool sortById = true;
+  stk::mesh::get_entities(bulk, rank, bulk.mesh_meta_data().locally_owned_part(), entities, sortById);
+
+  size_t numberBoundingBoxes = entities.size();
+  Kokkos::View<BoxIdentProcType *> boundingBoxes("Bounding Boxes", numberBoundingBoxes);
+  auto boundingBoxesHost = Kokkos::create_mirror_view(boundingBoxes);
+
+  stk::mesh::FieldBase const * coords = bulk.mesh_meta_data().coordinate_field();
+
+  std::vector<double> boxCoordinates(6);
+
+  for (size_t i = 0; i < entities.size(); ++i) {
+    unsigned num_nodes = bulk.num_nodes(entities[i]);
+    std::vector<double> coordinates(3*num_nodes,0);
+    const stk::mesh::Entity* nodes = bulk.begin_nodes(entities[i]);
+    for (unsigned j = 0; j < num_nodes; ++j) {
+      double* data = static_cast<double*>(stk::mesh::field_data(*coords, nodes[j]));
+      coordinates[3*j] = data[0];
+      coordinates[3*j+1] = data[1];
+      coordinates[3*j+2] = data[2];
+    }
+    findBoundingBoxCoordinates(coordinates, boxCoordinates);
+
+    int id = bulk.identifier(entities[i]);
+    IdentProcType domainBoxId;
+    if constexpr (std::is_same_v<IdentProcType, IdentProc>) {
+      domainBoxId = IdentProc(id, bulk.parallel_rank());
+    }
+    else {
+        domainBoxId = id;
+    }
+
+    boundingBoxesHost(i) = {BoxType(boxCoordinates[0], boxCoordinates[1], boxCoordinates[2],
+                                                 boxCoordinates[3], boxCoordinates[4], boxCoordinates[5]), domainBoxId};
+  }
+
+  Kokkos::deep_copy(boundingBoxes, boundingBoxesHost);
+
+  return boundingBoxes;
+}
+
+inline void createBoundingBoxesForElementsInElementBlocks(const stk::mesh::BulkData &bulk, FloatBoxIdentProcVector& domainBoxes)
+{
+  createBoundingBoxesForEntities(bulk, stk::topology::ELEM_RANK, domainBoxes);
+}
+
+inline void fillBoxesUsingElementBlocksFromFile(MPI_Comm comm, const std::string& volumeFilename, FloatBoxIdentProcVector &domainBoxes)
 {
     std::shared_ptr<stk::mesh::BulkData> bulk = build_mesh(3, comm);
     stk::io::fill_mesh(volumeFilename, *bulk);
@@ -358,7 +432,7 @@ inline void fillBoxesUsingElementBlocksFromFile(MPI_Comm comm, const std::string
 }
 
 inline void fillBoundingVolumesUsingNodesFromFile(
-        MPI_Comm comm, const std::string& sphereFilename, std::vector< std::pair<Sphere, Ident> > &spheres)
+        MPI_Comm comm, const std::string& sphereFilename, std::vector< std::pair<Sphere, IdentProc> > &spheres)
 {
     std::shared_ptr<stk::mesh::BulkData> bulk = build_mesh(3, comm);
     stk::mesh::MetaData& meta = bulk->mesh_meta_data();
@@ -385,11 +459,11 @@ inline void fillBoundingVolumesUsingNodesFromFile(
 
         double radius=1e-5;
         unsigned id = bulk->identifier(node);
-        spheres[i] = std::make_pair(Sphere(Point(x,y,z), radius), Ident(id, bulk->parallel_rank()));
+        spheres[i] = std::make_pair(Sphere(Point(x,y,z), radius), IdentProc(id, bulk->parallel_rank()));
     }
 }
 
-inline void fillBoundingVolumesUsingNodesFromFile(MPI_Comm comm, const std::string& sphereFilename, FloatBoxVector &spheres)
+inline void fillBoundingVolumesUsingNodesFromFile(MPI_Comm comm, const std::string& sphereFilename, FloatBoxIdentProcVector &spheres)
 {
     std::shared_ptr<stk::mesh::BulkData> bulk = build_mesh(3, comm);
     stk::mesh::MetaData& meta = bulk->mesh_meta_data();
@@ -417,7 +491,7 @@ inline void fillBoundingVolumesUsingNodesFromFile(MPI_Comm comm, const std::stri
         double radius=1e-5;
         unsigned id = bulk->identifier(node);
         FloatBox box(x-radius, y-radius, z-radius, x+radius, y+radius, z+radius);
-        spheres[i] = std::make_pair(box, Ident(id, bulk->parallel_rank()));
+        spheres[i] = std::make_pair(box, IdentProc(id, bulk->parallel_rank()));
     }
 }
 
@@ -426,7 +500,7 @@ namespace simple_fields {
 inline void findBoundingBoxCoordinates(const std::vector<double> &coordinates, std::vector<double>& boxCoordinates)
 {
     int spatialDim = 3;
-    double *minCoordinates = &boxCoordinates[0];
+    double *minCoordinates = boxCoordinates.data();
     double *maxCoordinates = &boxCoordinates[spatialDim];
 
     for (int j=0;j<spatialDim;j++)
@@ -593,7 +667,7 @@ inline void putCoordinatesInFile(const int exoid, const std::vector<FloatBox>& b
         double zmax = boxes[i].get_z_max();
 
         double hexCoordinates[24];
-        setHexCoordinates(xmin, ymin, zmin, xmax, ymax, zmax, &hexCoordinates[0]);
+        setHexCoordinates(xmin, ymin, zmin, xmax, ymax, zmax, hexCoordinates);
 
         unsigned offset = i*num_nodes_per_element;
         for (int j=0; j<num_nodes_per_element; j++)
@@ -655,7 +729,7 @@ inline void writeExodusFileUsingBoxes(const std::vector<FloatBox>& boxes, const
         }
         offset += num_elements_this_block*num_nodes_per_elem;
 
-        ex_put_elem_conn(exoid, blockId, &connect[0]);
+        ex_put_elem_conn(exoid, blockId, connect.data());
     }
 
     ex_close(exoid);
@@ -677,53 +751,23 @@ fillDomainBoxes(MPI_Comm comm)
     return domainBoxes;
 }
 
-inline void fillStkBoxesUsingFloatBoxes(const std::vector<FloatBox> &domainBoxes, const int procId, StkBoxVector& stkBoxes)
+inline void fillStkBoxesUsingFloatBoxes(const std::vector<FloatBox> &domainBoxes, const int procId, StkBoxIdentProcVector& stkBoxes)
 {
     for (size_t i=0;i<domainBoxes.size();i++)
     {
         Point min(domainBoxes[i].get_x_min(), domainBoxes[i].get_y_min(), domainBoxes[i].get_z_min());
         Point max(domainBoxes[i].get_x_max(), domainBoxes[i].get_y_max(), domainBoxes[i].get_z_max());
-        Ident domainBoxId(i, procId);
+        IdentProc domainBoxId(i, procId);
         stkBoxes[i] = std::make_pair(StkBox(min,max), domainBoxId);
     }
 }
 
-inline void createBoundingBoxesForElementsInElementBlocks(const stk::mesh::BulkData &bulk, FloatBoxVector& domainBoxes)
+inline void createBoundingBoxesForElementsInElementBlocks(const stk::mesh::BulkData &bulk, FloatBoxIdentProcVector& domainBoxes)
 {
-    stk::mesh::EntityVector elements;
-    const bool sortById = true;
-    stk::mesh::get_entities(bulk, stk::topology::ELEM_RANK, bulk.mesh_meta_data().locally_owned_part(), elements, sortById);
-
-    size_t numberBoundingBoxes = elements.size();
-    domainBoxes.resize(numberBoundingBoxes);
-
-    stk::mesh::FieldBase const * coords = bulk.mesh_meta_data().coordinate_field();
-
-    std::vector<double> boxCoordinates(6);
-
-    for(size_t i=0;i<elements.size();++i)
-    {
-        unsigned num_nodes = bulk.num_nodes(elements[i]);
-        std::vector<double> coordinates(3*num_nodes,0);
-        const stk::mesh::Entity* nodes = bulk.begin_nodes(elements[i]);
-        for(unsigned j=0;j<num_nodes;++j)
-        {
-            double* data = static_cast<double*>(stk::mesh::field_data(*coords, nodes[j]));
-            coordinates[3*j] = data[0];
-            coordinates[3*j+1] = data[1];
-            coordinates[3*j+2] = data[2];
-        }
-        findBoundingBoxCoordinates(coordinates, boxCoordinates);
-        unsigned id = bulk.identifier(elements[i]);
-        Ident domainBoxId(id, bulk.parallel_rank());
-        domainBoxes[i] = std::make_pair(FloatBox(boxCoordinates[0], boxCoordinates[1], boxCoordinates[2],
-                                                boxCoordinates[3], boxCoordinates[4], boxCoordinates[5]),
-                                                domainBoxId);
-
-    }
+  createBoundingBoxesForEntities(bulk, stk::topology::ELEM_RANK, domainBoxes);
 }
 
-inline void fillBoxesUsingElementBlocksFromFile(MPI_Comm comm, const std::string& volumeFilename, FloatBoxVector &domainBoxes)
+inline void fillBoxesUsingElementBlocksFromFile(MPI_Comm comm, const std::string& volumeFilename, FloatBoxIdentProcVector &domainBoxes)
 {
     std::shared_ptr<stk::mesh::BulkData> bulk = build_mesh(3, comm);
     stk::io::fill_mesh(volumeFilename, *bulk);
@@ -732,7 +776,7 @@ inline void fillBoxesUsingElementBlocksFromFile(MPI_Comm comm, const std::string
 }
 
 inline void fillBoundingVolumesUsingNodesFromFile(
-        MPI_Comm comm, const std::string& sphereFilename, std::vector< std::pair<Sphere, Ident> > &spheres)
+        MPI_Comm comm, const std::string& sphereFilename, std::vector< std::pair<Sphere, IdentProc> > &spheres)
 {
     std::shared_ptr<stk::mesh::BulkData> bulkPtr = build_mesh(3, comm);
     stk::mesh::MetaData& meta = bulkPtr->mesh_meta_data();
@@ -760,11 +804,11 @@ inline void fillBoundingVolumesUsingNodesFromFile(
 
         double radius=1e-5;
         unsigned id = bulk.identifier(node);
-        spheres[i] = std::make_pair(Sphere(Point(x,y,z), radius), Ident(id, bulk.parallel_rank()));
+        spheres[i] = std::make_pair(Sphere(Point(x,y,z), radius), IdentProc(id, bulk.parallel_rank()));
     }
 }
 
-inline void fillBoundingVolumesUsingNodesFromFile(MPI_Comm comm, const std::string& sphereFilename, FloatBoxVector &spheres)
+inline void fillBoundingVolumesUsingNodesFromFile(MPI_Comm comm, const std::string& sphereFilename, FloatBoxIdentProcVector &spheres)
 {
     std::shared_ptr<stk::mesh::BulkData> bulkPtr = build_mesh(3, comm);
     stk::mesh::MetaData& meta = bulkPtr->mesh_meta_data();
@@ -793,7 +837,7 @@ inline void fillBoundingVolumesUsingNodesFromFile(MPI_Comm comm, const std::stri
         double radius=1e-5;
         unsigned id = bulk.identifier(node);
         FloatBox box(x-radius, y-radius, z-radius, x+radius, y+radius, z+radius);
-        spheres[i] = std::make_pair(box, Ident(id, bulk.parallel_rank()));
+        spheres[i] = std::make_pair(box, IdentProc(id, bulk.parallel_rank()));
     }
 }
 
