@@ -293,6 +293,33 @@ TEUCHOS_UNIT_TEST(tDiagnosticPreconditionerFactory, inverse_lib_test_tpetra) {
   Teko::applyOp(invA, x, y);
 }
 
+TEUCHOS_UNIT_TEST(tDiagnosticPreconditionerFactory, inverse_lib_test_tpetra_preconditioned_solve) {
+  // build global (or serial communicator)
+  RCP<const Teuchos::Comm<int> > Comm = Tpetra::getDefaultComm();
+
+  // setup diagnostic inverse parameter list: uses Amesos under neady
+  Teuchos::ParameterList pl;
+  Teuchos::ParameterList& diagList = pl.sublist("Diagnostic");
+  diagList.set<std::string>("Type", "Diagnostic Inverse");
+  diagList.set<std::string>("Inverse Factory", "Belos");
+  diagList.set<std::string>("Preconditioner Factory", "Ifpack2");
+  diagList.set<std::string>("Descriptive Label", "the_descriptive_label");
+
+  // build inverse factory
+  RCP<Teko::InverseLibrary> invLibrary = Teko::InverseLibrary::buildFromParameterList(pl);
+  RCP<Teko::InverseFactory> invFact    = invLibrary->getInverseFactory("Diagnostic");
+
+  // build inverse operator
+  Teko::LinearOp A    = buildSystem(Comm, 2000);
+  Teko::LinearOp invA = Teko::buildInverse(*invFact, A);
+
+  // apply inverse
+  Teko::MultiVector x = Thyra::createMember(invA->domain());
+  Teko::MultiVector y = Thyra::createMember(invA->range());
+  Thyra::randomize(-1.0, 1.0, x.ptr());
+  Teko::applyOp(invA, x, y);
+}
+
 #ifdef TEKO_HAVE_EPETRA
 TEUCHOS_UNIT_TEST(tDiagnosticPreconditionerFactory, construction_test) {
 // build global (or serial communicator)
